import enum


class ObjectDetectionReturnTypes(enum.Enum):
    """Enum for the return types of the object detection functions."""
    # The object detection function returns a list of objects.
    LABELS = "LABELS"
    # The object detection function returns a dictionary of objects.
    BBOX = "BBOX"

    CONF = "CONF"

    ALL = "ALL"


class DroneCommands(enum.Enum):
    NONE = None
    TAKE_OFF = "start flying the drone",
    LAND = "land the drone",
    FORWARD = "move the drone forward",
    BACKWARD = "move the drone backward",
    LEFT = "move to the left",
    RIGHT = "move to the right",
    UP = "go up",
    DOWN = "go down",
    ROTATE_CLOCKWISE = "rotate the drone clockwise",
    ROTATE_COUNTER_CLOCKWISE = "rotate the drone counter clockwise",
    BATTERY = "BATTERY",
    SPEED = "SPEED",
    HEIGHT = "HEIGHT",
    STOP = "STOP",
    RECOGNIZE_PEOPLE = "recognize the people in the image",
    RECOGNIZE_OBJECTS = "recognize the objects in the image",
    RECOGNIZE_TEXT = "recognize a text from the image",
    FLIP = "do a flip",
    #  = "TURN_RIGHT_AND_RECOGNIZE_OBJECTS",
    FOLLOW_ME = "follow a person",
    LOCATE_OBJECTS_AND_RECOGNIZE = "find the objects and recognize them",
    MOVE_AROUND = "move around the room",
    TAKE_A_PHOTO = "take a photo",


class LoggerColors(enum.Enum):
    RED = "\033[0;31m"
    GREEN = "\033[0;32m"
    YELLOW = "\033[0;33m"
    BLUE = "\033[0;34m"
    PURPLE = "\033[0;35m"
    CYAN = "\033[0;36m"
    WHITE = "\033[0;37m"


class Language(enum.Enum):
    ENGLISH = 'en-gb',
    FRENCH = 'FR',


class VisionAlgorithm(enum.Enum):
    YOLO = 'YOLO',
    GOOGLE_VISION = 'GOOGLE_VISION',
    MEDIA_PIPE = 'MEDIA_PIPE',


class ObjectRecognitionAlgorithm(enum.Enum):
    YOLO_TRANSFER_LEARNING = 'YOLO_TRANSFER_LEARNING',


class SpeechGenerationAlgorithm(enum.Enum):
    GOOGLE_TTS_OFFLINE = 'GOOGLE_TTS_OFFLINE',


class IntentRecognitionAlgorithm(enum.Enum):
    CHAT_GPT = 'CHAT_GPT',
    SNIPS_NLU = 'SNIPS_NLU',


class FaceRecognitionAlgorithm(enum.Enum):
    FACE_RECOGNITION_EUCLIDEAN = 'FACE_RECOGNITION_EUCLIDEAN',
    FACE_RECOGNITION_KNN = 'FACE_RECOGNITION_KNN',


class PlaceRecognitionAlgorithm(enum.Enum):
    PLACE_RECOGNITION_KNN = 'PLACE_RECOGNITION_KNN',
    PLACE_RECOGNITION_RF = 'PLACE_RECOGNITION_RF',


class TextRecognitionAlgorithm(enum.Enum):
    GOOGLE_VISION = 'GOOGLE_VISION',


class AtomicEngineConfigurations(enum.Enum):
    # The configuration for the object detection algorithm.
    OBJECT_DETECTION_YOLO_V3_WEIGHTS_PATH = "OBJECT_DETECTION_YOLO_V3_WEIGHTS_PATH"
    OBJECT_DETECTION_YOLO_VERSION = "OBJECT_DETECTION_YOLO_VERSION"
    OBJECT_DETECTION_MP_MODELS_PATH = "OBJECT_DETECTION_MP_MODELS_PATH"

    # The configuration for the speech generation algorithm.

    SPEECH_GENERATION_TTS_RATE = "SPEECH_GENERATION_TTS_RATE"
    SPEECH_GENERATION_TTS_VOLUME = "SPEECH_GENERATION_TTS_VOLUME"
    SPEECH_GENERATION_TTS_VOICE_ID = "SPEECH_GENERATION_TTS_VOICE_ID"

    # The configuration for the intent recognition algorithm.

    INTENT_RECOGNITION_OPEN_AI_API_KEY = "INTENT_RECOGNITION_OPEN_AI_API_KEY"
    INTENT_RECOGNITION_OPEN_AI_MODEL = "INTENT_RECOGNITION_OPEN_AI_MODEL"
    INTENT_RECOGNITION_OPEN_AI_API_URL = "INTENT_RECOGNITION_OPEN_AI_API_URL"
    INTENT_RECOGNITION_OPEN_AI_TEMPERATURE = "INTENT_RECOGNITION_OPEN_AI_TEMPERATURE"
    INTENT_RECOGNITION_OPEN_AI_LOGGER_LOCATION = "INTENT_RECOGNITION_OPEN_AI_LOGGER_LOCATION"
    INTENT_RECOGNITION_OPEN_AI_SYSTEM_PROMPT = "INTENT_RECOGNITION_OPEN_AI_SYSTEM_PROMPT"
    INTENT_RECOGNITION_OPEN_AI_SYSTEM_ACTIONS_PATH = "INTENT_RECOGNITION_OPEN_AI_SYSTEM_ACTIONS_PATH"

    INTENT_RECOGNITION_SNIPS_NLU_DATASET_PATH = "INTENT_RECOGNITION_SNIPS_NLU_DATASET_PATH"
    INTENT_RECOGNITION_SNIPS_LANGUAGE_CONFIG = "INTENT_RECOGNITION_SNIPS_LANGUAGE_CONFIG"

    # The configuration for the speech recognition algorithm.
    SPEECH_RECOGNITION_VOSK_LANGUAGE = "SPEECH_RECOGNITION_VOSK_LANGUAGE"
    SPEECH_RECOGNITION_VOSK_LANGUAGE_MODEL_PATH = "SPEECH_RECOGNITION_VOSK_LANGUAGE_MODEL_PATH"

    SPEECH_RECOGNITION_GOOGLE_LANGUAGE_CODE = "SPEECH_RECOGNITION_GOOGLE_LANGUAGE_CODE"
    SPEECH_RECOGNITION_GOOGLE_SAMPLE_RATE_HERTZ = "SPEECH_RECOGNITION_GOOGLE_SAMPLE_RATE_HERTZ"
    SPEECH_RECOGNITION_GOOGLE_ENCODING = "SPEECH_RECOGNITION_GOOGLE_ENCODING"

    SPEECH_RECOGNITION_MULTI_ALGO_ALGORITHM_NAME = "SPEECH_RECOGNITION_MULTI_ALGO_ALGORITHM_NAME"
    SPEECH_RECOGNITION_MULTI_ALGO_ALGO_MIC_TIMEOUT = "SPEECH_RECOGNITION_MULTI_ALGO_ALGO_MIC_TIMEOUT"
    SPEECH_RECOGNITION_MULTI_ALGO_ALGO_PHRASE_TIME_LIMIT = "SPEECH_RECOGNITION_MULTI_ALGO_ALGO_PHRASE_TIME_LIMIT"
    SPEECH_RECOGNITION_MULTI_ALGO_IBM_KEY = "SPEECH_RECOGNITION_MULTI_ALGO_IBM_KEY"

    HAND_FEATURE_EXTRACTION_ENABLE_GESTURE_RECOGNITION = "HAND_FEATURE_EXTRACTION_ENABLE_GESTURE_RECOGNITION"
    HAND_FEATURE_EXTRACTION_GESTURE_RECOGNITION_MODEL_PATH = "HAND_FEATURE_EXTRACTION_GESTURE_RECOGNITION_MODEL_PATH"

    BODY_FEATURE_EXTRACTION_POSTURE_DETECTION_MODEL_PATH = "BODY_FEATURE_EXTRACTION_POSTURE_DETECTION_MODEL_PATH"

    FACE_RECOGNITION_KNN_TRAINING_DATA_SET_SIZE = "FACE_RECOGNITION_KNN_TRAINING_DATA_SET_SIZE"
    FACE_RECOGNITION_KNN_TESTING_DATA_SET_SIZE = "FACE_RECOGNITION_KNN_TESTING_DATA_SET_SIZE"
    FACE_RECOGNITION_KNN_VALIDATION_DATA_SET_SIZE = "FACE_RECOGNITION_KNN_VALIDATION_DATA_SET_SIZE"
    FACE_RECOGNITION_KNN_ALGORITHM_NAME = "FACE_RECOGNITION_KNN_ALGORITHM_NAME"
    FACE_RECOGNITION_KNN_ALGORITHM_NEIGHBORS = "FACE_RECOGNITION_KNN_ALGORITHM_NEIGHBORS"
    FACE_RECOGNITION_KNN_MODEL_SAVING_PATH = "FACE_RECOGNITION_KNN_MODEL_SAVING_PATH"
    FACE_RECOGNITION_KNN_MODEL_THRESHOLD = "FACE_RECOGNITION_KNN_MODEL_THRESHOLD"
    FACE_RECOGNITION_KNN_USE_DRONE_TO_CREATE_DATASET = "FACE_RECOGNITION_KNN_USE_DRONE_TO_CREATE_DATASET"
    FACE_RECOGNITION_KNN_DRONE_INSTANCE = "FACE_RECOGNITION_KNN_DRONE_INSTANCE"

    PLACE_RECOGNITION_KNN_TRAINING_DATA_SET_PATH = "PLACE_RECOGNITION_KNN_TRAINING_DATA_SET_PATH"
    PLACE_RECOGNITION_KNN_TESTING_DATA_SET_PATH = "PLACE_RECOGNITION_KNN_TESTING_DATA_SET_PATH"
    PLACE_RECOGNITION_KNN_VALIDATION_DATA_SET_PATH = "PLACE_RECOGNITION_KNN_VALIDATION_DATA_SET_PATH"
    PLACE_RECOGNITION_KNN_ALGORITHM_NAME = "PLACE_RECOGNITION_KNN_ALGORITHM_NAME"
    PLACE_RECOGNITION_KNN_ALGORITHM_NEIGHBORS = "PLACE_RECOGNITION_KNN_ALGORITHM_NEIGHBORS"
    PLACE_RECOGNITION_KNN_MODEL_SAVING_PATH = "PLACE_RECOGNITION_KNN_MODEL_SAVING_PATH"
    PLACE_RECOGNITION_KNN_MODEL_THRESHOLD = "PLACE_RECOGNITION_KNN_MODEL_THRESHOLD"
    PLACE_RECOGNITION_KNN_USE_DRONE_TO_CREATE_DATASET = "PLACE_RECOGNITION_KNN_USE_DRONE_TO_CREATE_DATASET"
    PLACE_RECOGNITION_KNN_DRONE_INSTANCE = "PLACE_RECOGNITION_KNN_DRONE_INSTANCE"
    PLACE_RECOGNITION_KNN_START_TRAINING_CALLBACK = "PLACE_RECOGNITION_KNN_START_TRAINING_CALLBACK"
    PLACE_RECOGNITION_KNN_END_TRAINING_CALLBACK = "PLACE_RECOGNITION_KNN_END_TRAINING_CALLBACK"
    PLACE_RECOGNITION_KNN_ALGORITHM_WEIGHTS = "PLACE_RECOGNITION_KNN_ALGORITHM_WEIGHTS"
    PLACE_RECOGNITION_KNN_CLASSIFIER_LOCATION = "PLACE_RECOGNITION_KNN_CLASSIFIER_LOCATION"
    PLACE_RECOGNITION_KNN_EXTRACTOR = "PLACE_RECOGNITION_KNN_EXTRACTOR"

    PLACE_RECOGNITION_RF_TRAINING_DATA_SET_PATH = "PLACE_RECOGNITION_RF_TRAINING_DATA_SET_PATH"
    PLACE_RECOGNITION_RF_TESTING_DATA_SET_PATH = "PLACE_RECOGNITION_RF_TESTING_DATA_SET_PATH"
    PLACE_RECOGNITION_RF_VALIDATION_DATA_SET_PATH = "PLACE_RECOGNITION_RF_VALIDATION_DATA_SET_PATH"
    PLACE_RECOGNITION_RF_ALGORITHM_NAME = "PLACE_RECOGNITION_RF_ALGORITHM_NAME"
    PLACE_RECOGNITION_RF_ALGORITHM_NEIGHBORS = "PLACE_RECOGNITION_RF_ALGORITHM_NEIGHBORS"
    PLACE_RECOGNITION_RF_MODEL_SAVING_PATH = "PLACE_RECOGNITION_RF_MODEL_SAVING_PATH"
    PLACE_RECOGNITION_RF_MODEL_THRESHOLD = "PLACE_RECOGNITION_RF_MODEL_THRESHOLD"
    PLACE_RECOGNITION_RF_USE_DRONE_TO_CREATE_DATASET = "PLACE_RECOGNITION_RF_USE_DRONE_TO_CREATE_DATASET"
    PLACE_RECOGNITION_RF_DRONE_INSTANCE = "PLACE_RECOGNITION_RF_DRONE_INSTANCE"
    PLACE_RECOGNITION_RF_START_TRAINING_CALLBACK = "PLACE_RECOGNITION_RF_START_TRAINING_CALLBACK"
    PLACE_RECOGNITION_RF_END_TRAINING_CALLBACK = "PLACE_RECOGNITION_RF_END_TRAINING_CALLBACK"
    PLACE_RECOGNITION_RF_ALGORITHM_WEIGHTS = "PLACE_RECOGNITION_RF_ALGORITHM_WEIGHTS"
    PLACE_RECOGNITION_RF_CLASSIFIER_LOCATION = "PLACE_RECOGNITION_RF_CLASSIFIER_LOCATION"
    PLACE_RECOGNITION_RF_EXTRACTOR = "PLACE_RECOGNITION_RF_EXTRACTOR"

    OBJECT_IDENTIFICATION_YOLO_WEIGHTS_PATH = "OBJECT_IDENTIFICATION_YOLO_WEIGHTS_PATH"
    OBJECT_IDENTIFICATION_YOLO_DRONE_INSTANCE = "OBJECT_IDENTIFICATION_YOLO_DRONE_INSTANCE"
    OBJECT_IDENTIFICATION_KNN_ALGORITHM_NAME = "OBJECT_IDENTIFICATION_KNN_ALGORITHM_NAME"
    OBJECT_IDENTIFICATION_KNN_ALGORITHM_NEIGHBORS = "OBJECT_IDENTIFICATION_KNN_ALGORITHM_NEIGHBORS"
    OBJECT_IDENTIFICATION_KNN_MODEL_SAVING_PATH = "OBJECT_IDENTIFICATION_KNN_MODEL_SAVING_PATH"
    OBJECT_IDENTIFICATION_KNN_MODEL_THRESHOLD = "OBJECT_IDENTIFICATION_KNN_MODEL_THRESHOLD"
    OBJECT_IDENTIFICATION_KNN_USE_DRONE_TO_CREATE_DATASET = "OBJECT_IDENTIFICATION_KNN_USE_DRONE_TO_CREATE_DATASET"
    OBJECT_IDENTIFICATION_KNN_DRONE_INSTANCE = "OBJECT_IDENTIFICATION_KNN_DRONE_INSTANCE"
    OBJECT_IDENTIFICATION_KNN_TRAINING_DATA_SET_SIZE = "OBJECT_IDENTIFICATION_KNN_TRAINING_DATA_SET_SIZE"
    OBJECT_IDENTIFICATION_KNN_TESTING_DATA_SET_SIZE = "OBJECT_IDENTIFICATION_KNN_TESTING_DATA_SET_SIZE"
    OBJECT_IDENTIFICATION_KNN_VALIDATION_DATA_SET_SIZE = "OBJECT_IDENTIFICATION_KNN_VALIDATION_DATA_SET_SIZE"
    OBJECT_IDENTIFICATION_KNN_EXTRACTOR = "OBJECT_IDENTIFICATION_KNN_EXTRACTOR"
    OBJECT_IDENTIFICATION_KNN_START_TRAINING_CALLBACK = "OBJECT_IDENTIFICATION_KNN_START_TRAINING_CALLBACK"
    OBJECT_IDENTIFICATION_KNN_END_TRAINING_CALLBACK = "OBJECT_IDENTIFICATION_KNN_END_TRAINING_CALLBACK"
    OBJECT_IDENTIFICATION_KNN_ALGORITHM_WEIGHTS = "OBJECT_IDENTIFICATION_KNN_ALGORITHM_WEIGHTS"
    OBJECT_IDENTIFICATION_KNN_CLASSIFIER_LOCATION = "OBJECT_IDENTIFICATION_KNN_CLASSIFIER_LOCATION"
    OBJECT_IDENTIFICATION_KNN_MODEL_PATH = "OBJECT_IDENTIFICATION_KNN_MODEL_PATH"
    OBJECT_IDENTIFICATION_KNN_MODEL_NAME = "OBJECT_IDENTIFICATION_KNN_MODEL_NAME"
    OBJECT_IDENTIFICATION_KNN_MODEL_VERSION = "OBJECT_IDENTIFICATION_KNN_MODEL_VERSION"

    OBJECT_IDENTIFICATION_GPT_API_KEY = "OBJECT_IDENTIFICATION_GPT_API_KEY"
    OBJECT_IDENTIFICATION_GPT_MODEL = "OBJECT_IDENTIFICATION_GPT_MODEL"

    OBJECT_IDENTIFICATION_SIAMESE_YOLO_VERSION = "OBJECT_IDENTIFICATION_SIAMESE_YOLO_VERSION"
